import json
import jsonschema

CONFIG_SCHEMA = {
    "$schema": "http://json-schema.org/draft-07/schema#",

    "type": "object",
    "properties": {
        "admin_username": {"type": "string"},
        "admin_password": {"type": "string"},
        "hostname": {"type": "string"},
        "listen_port": {"type": "integer"},
        "grafana_port": {"type": "integer"},
        "organizations": {"type": "array", "items": {"type": "string"}},
        "BRIAN_ENVIRONMENT": {"type": "string"}
    },
    "required": ["admin_username", "admin_password", "hostname", "listen_port", "grafana_port", "organizations", "BRIAN_ENVIRONMENT"],
    "additionalProperties": False
}


def defaults():
    return {
        "admin_username": "admin",
        "admin_password": "admin",
        "hostname": "localhost",
        "listen_port": 3001,
        "grafana_port": 3000,
        "organizations": ["Main Org."],
        "BRIAN_ENVIRONMENT": "test"
    }


def load(f):
    """
    loads, validates and returns configuration parameters

    :param f: file-like object that produces the config file
    :return:
    """
    config = json.loads(f.read())
    jsonschema.validate(config, CONFIG_SCHEMA)
    return config
