import logging
import os
import json
from functools import reduce
from typing import List
from brian_dashboard_manager.grafana.utils.request import AdminRequest, TokenRequest
from brian_dashboard_manager.grafana.organization import get_organizations, create_organization, delete_organization, create_api_token, delete_api_token, delete_expired_api_tokens
from brian_dashboard_manager.grafana.dashboard import provision_dashboard


logger = logging.getLogger(__name__)


def provision(config):

    hostname = config.get('hostname')
    port = config.get('grafana_port')
    username = config.get('admin_username')
    password = config.get('admin_password')

    request = AdminRequest(hostname, port, username, password)
    all_orgs = get_organizations(request)

    organizations_to_provision = config.get('organizations', [])

    missing = [name for name in organizations_to_provision
               if name not in [org['name'] for org in all_orgs]]

    for org in missing:
        org_data = create_organization(request, org)
        all_orgs.append(org_data)

    for org in all_orgs:
        org_id = org['id']
        delete_expired_api_tokens(request, org_id)
        token = create_api_token(request, org_id)
        token_request = TokenRequest(hostname, port, token['key'])
        # TODO: (de)provision datasources and remove all existing dashboards in an organization before attempting to provision new dashboards.

        for (dirpath, dirnames, filenames) in os.walk(config.get('dashboard_directory', 'dashboards')):
            for file in filenames:
                if file.endswith('.json'):
                    filename = os.path.join(dirpath, file)
                    logger.info(f'Provisioning dashboard: {file.strip(".json")}')
                    provision_dashboard(token_request, json.load(open(filename, 'r')))

        delete_api_token(request, org_id, token['id'])

    return all_orgs
