import requests
import json


class Request(object):
    def __init__(self, url, headers=None):
        self.headers = {
            'Accept': 'application/json'
        }
        if headers:
            self.headers.update(headers)

        self.BASE_URL = url

    def get(self, endpoint: str, headers=None, **kwargs):

        r = requests.get(
            self.BASE_URL + endpoint,
            headers={**headers, **self.headers} if headers else self.headers,
            **kwargs
        )
        r.raise_for_status()
        try:
            return r.json()
        except json.JSONDecodeError:
            return None

    def post(self, endpoint: str, headers=None, **kwargs):

        r = requests.post(
            self.BASE_URL + endpoint,
            headers={**headers, **self.headers} if headers else self.headers,
            **kwargs
        )
        r.raise_for_status()
        try:
            return r.json()
        except json.JSONDecodeError:
            return None

    def delete(self, endpoint: str, headers=None, **kwargs):

        r = requests.delete(
            self.BASE_URL + endpoint,
            headers={**headers, **self.headers} if headers else self.headers,
            **kwargs
        )
        r.raise_for_status()
        try:
            return r.json()
        except json.JSONDecodeError:
            return None


class AdminRequest(Request):
    def __init__(self, hostname, grafana_port, admin_username, admin_password, **kwargs):
        self.username = admin_username
        super().__init__(
            f'http://{admin_username}:{admin_password}@{hostname}:{grafana_port}/')

    def __str__(self):
        return f'admin user: {self.username}'


class TokenRequest(Request):
    def __init__(self, hostname, grafana_port, token: str, **kwargs):
        self.token = token

        super().__init__(f'http://{hostname}:{grafana_port}/', {
            'Authorization': 'Bearer ' + token
        })

    def __str__(self):
        return f'token {self.token}'
